#define WIN32_LEAN_AND_MEAN
#include "windows.h"

#pragma pack(1)
typedef struct server_info_50 {
   char sv50_name[ 16 ];
   unsigned char sv50_version_major;
   unsigned char sv50_version_minor;
   unsigned long sv50_type;
   char *sv50_comment;
   unsigned short sv50_security;
   unsigned short sv50_auditing;
   char *sv50_container;
   char *sv50_ab_server;
   char *sv50_ab_dll;
} SERVER_INFO_50, * PSERVER_INFO_50;

//  sv50_type defines
#define SV_TYPE_WORKSTATION 0x00000001
#define SV_TYPE_SERVER 0x00000002
#define SV_TYPE_SQLSERVER 0x00000004
#define SV_TYPE_DOMAIN_CTRL 0x00000008
#define SV_TYPE_DOMAIN_BAKCTRL 0x00000010
#define SV_TYPE_TIME_SOURCE 0x00000020
#define SV_TYPE_AFP 0x00000040
#define SV_TYPE_NOVELL 0x00000080
#define SV_TYPE_DOMAIN_MEMBER 0x00000100
#define SV_TYPE_PRINTQ_SERVER 0x00000200
#define SV_TYPE_DIALIN_SERVER 0x00000400
#define SV_TYPE_ALL 0xFFFFFFFF

//  Defines for sv50_security
#define SV_USERSECURITY 1
#define SV_SHARESECURITY 0

#define SV_SECURITY_SHARE 0
#define SV_SECURITY_WINNT 1
#define SV_SECURITY_WINNTAS 2
#define SV_SECURITY_NETWARE 3

typedef struct _SERVER_INFO_101 {
    DWORD           sv101_platform_id;
    LPTSTR          sv101_name;
    DWORD           sv101_version_major;
    DWORD           sv101_version_minor;
    DWORD           sv101_type;
    LPTSTR          sv101_comment;
} SERVER_INFO_101, *PSERVER_INFO_101, *LPSERVER_INFO_101;


BOOL WINAPI MainDlgProc( HWND, UINT, WPARAM, LPARAM );
BOOL GetServerInfo( HWND );
BOOL GetServerInfo95( HWND );
BOOL GetServerInfoNT( HWND );


HINSTANCE hInst;


int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE hPrev, LPSTR lpCmd,
                    int nShow )
{
   hInst = hInstance;
   DialogBox( hInstance, MAKEINTRESOURCE( 10000 ), NULL, MainDlgProc );
   return( TRUE );
}


BOOL WINAPI MainDlgProc( HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam )
{
   switch( msg )
   {
      case WM_INITDIALOG:
         return( TRUE );


      case WM_COMMAND:
         if( wParam == IDCANCEL )
         {
            EndDialog( hDlg, TRUE );
            return( TRUE );
         }
         if( wParam == IDOK )
         {
            GetServerInfo( hDlg );
            break;
         }
         break;
   }
   return( FALSE );
}


BOOL GetServerInfo( HWND hDlg )
{
   OSVERSIONINFO osvi;

   osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
   GetVersionEx( &osvi );
   if( osvi.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS )
      return( GetServerInfo95( hDlg ) );
   else
      return( GetServerInfoNT( hDlg ) );
}


BOOL GetServerInfo95( HWND hDlg )
{
   PSERVER_INFO_50 psi50;
   char szServer[ 32 ], szBuffer[ 128 ];
   int iStatus;
   USHORT usTotal;
   HANDLE hModule;
   int ( CALLBACK *pNetServerGetInfo )( LPSTR, short, LPVOID, USHORT, USHORT * );


   if( ! GetDlgItemText( hDlg, 101, szServer, 32 ) )
      return( FALSE );

   hModule = LoadLibrary( "SVRAPI.DLL" );
   if( ! hModule )
      return( FALSE );

   (FARPROC)pNetServerGetInfo = (FARPROC)GetProcAddress( hModule,
                                                         "NetServerGetInfo" );
   if( ! pNetServerGetInfo )
   {
      FreeLibrary( hModule );
      return( FALSE );
   }

   psi50 = (PSERVER_INFO_50)szBuffer;
   memset( psi50, 0, 128 );
   iStatus = pNetServerGetInfo( szServer, 50, psi50, 128, &usTotal );
   if( ! iStatus )
   {
      SendDlgItemMessage( hDlg, 100, LB_RESETCONTENT, 0, 0 );

      SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                          (LPARAM)psi50->sv50_name );
      wsprintf( szServer, "WinNet Version %d.%d", psi50->sv50_version_major,
                psi50->sv50_version_minor );
      SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, (LPARAM)szServer );

      if( psi50->sv50_type & SV_TYPE_WORKSTATION )
         SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                             (LPARAM)"  Workstation" );
      if( psi50->sv50_type & SV_TYPE_SERVER )
         SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                             (LPARAM)"  Server" );
      if( psi50->sv50_type & SV_TYPE_SQLSERVER )
         SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                             (LPARAM)"  SQL Server" );
      if( psi50->sv50_type & SV_TYPE_DOMAIN_CTRL )
         SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                             (LPARAM)"  Domain Controller" );
      if( psi50->sv50_type & SV_TYPE_DOMAIN_BAKCTRL )
         SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                             (LPARAM)"  Domain Backup" );

      if( psi50->sv50_comment )
         if( strlen( psi50->sv50_comment ) )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0,
                                (LPARAM)psi50->sv50_comment );

      if( psi50->sv50_container )
         if( strlen( psi50->sv50_container ) )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0,
                                (LPARAM)psi50->sv50_container );
   }
   FreeLibrary( hModule );
   return( TRUE );
}



BOOL GetServerInfoNT( HWND hDlg )
{
   DWORD ( CALLBACK *pNetServerGetInfo )( LPTSTR, DWORD, LPBYTE );
   DWORD dwStatus;
   HANDLE hModule;
   PSERVER_INFO_101 psv101;
   BYTE bBuffer[ 256 ];
   char szServer[ 32 ], szWServer[ 64 ], szComment[ 128 ];

   if( ! GetDlgItemText( hDlg, 101, szServer, 32 ) )
      return( FALSE );

   MultiByteToWideChar( CP_ACP, MB_COMPOSITE, szServer, -1, (USHORT *)szWServer, 64 );

   hModule = LoadLibrary( "NETAPI32.DLL" );
   if( ! hModule )
      return( FALSE );

   (FARPROC)pNetServerGetInfo = (FARPROC)GetProcAddress( hModule,
                                                         "NetServerGetInfo" );
   if( ! pNetServerGetInfo )
   {
      FreeLibrary( hModule );
      return( FALSE );
   }

   psv101 = (PSERVER_INFO_101)bBuffer;
   memset( psv101, 0, 256 );

   dwStatus = pNetServerGetInfo( szWServer, 101, (LPBYTE)psv101 );
   if( ! dwStatus )
   {
      SendDlgItemMessage( hDlg, 100, LB_RESETCONTENT, 0, 0 );
      psv101 = (PSERVER_INFO_101)psv101->sv101_platform_id;
      if( psv101->sv101_name )
      {
         WideCharToMultiByte( CP_ACP, 0, psv101->sv101_name, -1, szServer, 32,
                              NULL, NULL );
         SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, (LPARAM)szServer );

         wsprintf( szServer, "WinNet Version %d.%d", psv101->sv101_version_major,
                   psv101->sv101_version_minor );
         SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, (LPARAM)szServer );

         if( psv101->sv101_type & SV_TYPE_WORKSTATION )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"  Workstation" );
         if( psv101->sv101_type & SV_TYPE_SERVER )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"  Server" );
         if( psv101->sv101_type & SV_TYPE_SQLSERVER )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"  SQL Server" );
         if( psv101->sv101_type & SV_TYPE_DOMAIN_CTRL )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"  Domain Controller" );
         if( psv101->sv101_type & SV_TYPE_DOMAIN_BAKCTRL )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"  Domain Backup" );

         if( psv101->sv101_comment )
         {
            WideCharToMultiByte( CP_ACP, 0, psv101->sv101_comment, -1, szComment, 128,
                                 NULL, NULL );
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, (LPARAM)szComment );
         }
      }
   }

   FreeLibrary( hModule );
   return( TRUE );
}
